/*
 * Copyright 2010-2012 Craig S. Dickson (http://craigsdickson.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.squeakysand.sling.taglib;

import com.squeakysand.jcr.taglib.JcrSimpleTagSupport;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.jcr.Node;

import org.apache.sling.api.SlingHttpServletRequest;
import org.apache.sling.api.SlingHttpServletResponse;
import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ResourceResolver;
import org.apache.sling.api.scripting.SlingBindings;
import org.apache.sling.api.scripting.SlingScriptHelper;
import org.slf4j.Logger;

/**
 * Base class for writing JSP Tags that work with Apache Sling.
 * 
 * This tag makes all of the attributes set by the <sling:defineObjects/> tag available to tag developers.
 */
public abstract class SlingSimpleTagSupport extends JcrSimpleTagSupport {

    private static final String CURRENT_NODE_ATTRIBUTE_NAME = "currentNode";

    private static final String LOG_ATTRIBUTE_NAME = "log";

    private static final String RESOURCE_ATTRIBUTE_NAME = "resource";

    private static final String RESOURCE_RESOLVER_ATTRIBUTE_NAME = "resourceResolver";

    private static final String SLING_ATTRIBUTE_NAME = "sling";

    private static final String SLING_BINDINGS_ATTRIBUTE_NAME = "bindings";

    private static final String SLING_REQUEST_ATTRIBUTE_NAME = "slingRequest";

    private static final String SLING_RESPONSE_ATTRIBUTE_NAME = "slingResponse";

    protected Node getCurrentNode() {
        return getAttribute(CURRENT_NODE_ATTRIBUTE_NAME, Node.class);
    }

    protected Resource getResource() {
        return getAttribute(RESOURCE_ATTRIBUTE_NAME, Resource.class);
    }

    protected ResourceResolver getResourceResolver() {
        return getAttribute(RESOURCE_RESOLVER_ATTRIBUTE_NAME, ResourceResolver.class);
    }

    protected <T> T getService(Class<T> serviceType) {
        SlingScriptHelper slingScriptHelper = getSlingScriptHelper();
        return slingScriptHelper.getService(serviceType);
    }

    protected <T> List<T> getServices(Class<T> serviceType, String filter) {
        List<T> result = null;
        SlingScriptHelper slingScriptHelper = getSlingScriptHelper();
        T[] serviceArray = slingScriptHelper.getServices(serviceType, filter);
        if ((serviceArray == null) || (serviceArray.length < 1)) {
            result = new ArrayList<T>();
        } else {
            result = Arrays.asList(serviceArray);
        }
        return result;
    }

    protected SlingBindings getSlingBindings() {
        return getAttribute(SLING_BINDINGS_ATTRIBUTE_NAME, SlingBindings.class);
    }

    protected Logger getSlingLogger() {
        return getAttribute(LOG_ATTRIBUTE_NAME, Logger.class);
    }

    protected SlingHttpServletRequest getSlingRequest() {
        return getAttribute(SLING_REQUEST_ATTRIBUTE_NAME, SlingHttpServletRequest.class);
    }

    protected SlingHttpServletResponse getSlingResponse() {
        return getAttribute(SLING_RESPONSE_ATTRIBUTE_NAME, SlingHttpServletResponse.class);
    }

    protected SlingScriptHelper getSlingScriptHelper() {
        return getAttribute(SLING_ATTRIBUTE_NAME, SlingScriptHelper.class);
    }

    protected Resource resolveResource(Resource resource, String relativePath) {
        String absolutePath = resource.getPath() + relativePath;
        return resolveResource(absolutePath);
    }

    protected Resource resolveResource(String absolutePath) {
        return getResourceResolver().resolve(absolutePath);
    }

}
