package com.squeakysand.sling.taglib.properties;

import com.squeakysand.commons.lang.StringUtils;
import com.squeakysand.jsp.tagext.BodyContent;
import com.squeakysand.jsp.tagext.VariableScope;
import com.squeakysand.jsp.tagext.annotations.JspTag;
import com.squeakysand.jsp.tagext.annotations.JspTagAttribute;
import com.squeakysand.jsp.tagext.annotations.JspTagVariable;
import com.squeakysand.sling.taglib.SlingSimpleTagSupport;

import java.io.IOException;

import javax.servlet.jsp.JspException;

import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ResourceResolver;
import org.apache.sling.api.resource.ResourceUtil;
import org.apache.sling.api.resource.ValueMap;

/**
 * Selectively executes the body of the tag depending on whether or not the value of specified property of the specified
 * resource is <code>null</code> or not <code>null</code>.
 * 
 * @author <a href="http://craigsdickson.com">Craig S. Dickson</a>
 */
@JspTag(bodyContent = BodyContent.SCRIPTLESS)
public class IfPropertyTag extends SlingSimpleTagSupport {

    private String isNotNull;
    private String isNull;
    private String var;
    private String resourcePath;

    /**
     * @see javax.servlet.jsp.tagext.SimpleTagSupport#doTag()
     */
    @Override
    public void doTag() throws JspException, IOException {
        validateAttributeCombination("isNotNull", isNotNull, "isNull", isNull, AttributeCombination.EXACTLY_ONE);
        boolean isNotNullFlag = StringUtils.isNotNullOrEmpty(isNotNull) ? true : false;
        Resource resource = getResource();
        if (StringUtils.isNotNullOrEmpty(resourcePath)) {
            ResourceResolver resourceResolver = getResourceResolver();
            resource = resourceResolver.getResource(resource, resourcePath);
        }
        ValueMap properties = ResourceUtil.getValueMap(resource);
        String name = isNotNullFlag ? isNotNull : isNull;
        Object value = properties.get(name, null);
        if ((isNotNullFlag && (value != null)) || (!isNotNullFlag && (value == null))) {
            setPageAttribute(var, value);
            getJspBody().invoke(null);
        }
    }

    /**
     * Accessor for the <code>isNotNull</code> property.
     * 
     * @return the current value of the <code>isNotNull</code> property.
     */
    public String getIsNotNull() {
        return isNotNull;
    }

    /**
     * Accessor for the <code>isNull</code> property.
     * 
     * @return the current value of the <code>isNull</code> property.
     */
    public String getIsNull() {
        return isNull;
    }

    /**
     * Accessor for the <code>resourcePath</code> property.
     * 
     * @return the current value of the <code>resourcePath</code> property.
     */
    public String getResourcePath() {
        return resourcePath;
    }

    /**
     * Accessor for the <code>var</code> property.
     * 
     * @return the current value of the <code>var</code> property.
     */
    public String getVar() {
        return var;
    }

    /**
     * The name of the property to check the value is not <code>null</code>. If the value of the property is not
     * <code>null</code>, the body of the tag will be executed.
     * 
     * @param isNotNull
     *            the new value for the <code>isNotNull</code> property.
     */
    @JspTagAttribute(rtexprvalue = true)
    public void setIsNotNull(String isNotNull) {
        this.isNotNull = isNotNull;
    }

    /**
     * The name of the property to check the value is <code>null</code>. If the value of the property is
     * <code>null</code>, the body of the tag will be executed.
     * 
     * @param isNull
     *            the new value for the <code>isNull</code> property.
     */
    @JspTagAttribute(rtexprvalue = true)
    public void setIsNull(String isNull) {
        this.isNull = isNull;
    }

    /**
     * The path to the resource which has the property of interest. If this is not set or is <code>null</code> or is
     * empty, then the current resource in the JSP page context will be used, otherwise if this path is relative, it
     * will be resolved against the current resource in the JSP page context.
     * 
     * @param resourcePath
     *            the new value for the <code>resourcePath</code> property.
     */
    @JspTagAttribute(rtexprvalue = true)
    public void setResourcePath(String resourcePath) {
        this.resourcePath = resourcePath;
    }

    /**
     * The name of the page attribute that will be set with the value of the property, which can then be accessed using
     * EL within the body of the tag.
     * 
     * @param var
     *            the new value for the <code>var</code> property.
     */
    @JspTagVariable(scope = VariableScope.NESTED)
    @JspTagAttribute
    public void setVar(String var) {
        this.var = var;
    }

}
