package com.squeakysand.sling.taglib.properties;

import com.squeakysand.commons.lang.StringUtils;
import com.squeakysand.jsp.tagext.BodyContent;
import com.squeakysand.jsp.tagext.VariableScope;
import com.squeakysand.jsp.tagext.annotations.JspTag;
import com.squeakysand.jsp.tagext.annotations.JspTagAttribute;
import com.squeakysand.jsp.tagext.annotations.JspTagVariable;
import com.squeakysand.sling.taglib.SlingSimpleTagSupport;

import java.io.IOException;

import javax.servlet.jsp.JspException;

import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ResourceResolver;
import org.apache.sling.api.resource.ResourceUtil;
import org.apache.sling.api.resource.ValueMap;

/**
 * Accesses the value of the specified property and injects it into the JSP page context.
 * 
 * @author <a href="http://craigsdickson.com">Craig S. Dickson</a>
 */
@JspTag(bodyContent = BodyContent.EMPTY)
public class PropertyTag extends SlingSimpleTagSupport {

    private String name;
    private String var;
    private String resourcePath;

    /**
     * @see javax.servlet.jsp.tagext.SimpleTagSupport#doTag()
     */
    @Override
    public void doTag() throws JspException, IOException {
        Resource resource = getResource();
        if (StringUtils.isNotNullOrEmpty(resourcePath)) {
            ResourceResolver resourceResolver = getResourceResolver();
            resource = resourceResolver.getResource(resource, resourcePath);
        }
        ValueMap properties = ResourceUtil.getValueMap(resource);
        Object value = properties.get(name, null);
        setPageAttribute(var, value);
    }

    /**
     * Accessor for the <code>name</code> property.
     * 
     * @return the current value of the <code>name</code> property.
     */
    public String getName() {
        return name;
    }

    /**
     * Accessor for the <code>resourcePath</code> property.
     * 
     * @return the current value of the <code>resourcePath</code> property.
     */
    public String getResourcePath() {
        return resourcePath;
    }

    /**
     * Accessor for the <code>var</code> property.
     * 
     * @return the current value of the <code>var</code> property.
     */
    public String getVar() {
        return var;
    }

    /**
     * The simple name of the property to get the value from.
     * 
     * @param name
     *            the new value for the <code>name</code> property.
     */
    @JspTagAttribute(required = true, rtexprvalue = true)
    public void setName(String name) {
        this.name = name;
    }

    /**
     * The path to the resource which has the property of interest. If this is not set or is <code>null</code> or is
     * empty, then the current resource in the JSP page context will be used, otherwise if this path is relative, it
     * will be resolved against the current resource in the JSP page context.
     * 
     * @param resourcePath
     *            the new value for the <code>resourcePath</code> property.
     */
    @JspTagAttribute(rtexprvalue = true)
    public void setResourcePath(String resourcePath) {
        this.resourcePath = resourcePath;
    }

    /**
     * The name of the page attribute that will be set with the value of the property, which can then be accessed using
     * EL within the body of the tag.
     * 
     * @param var
     *            the new value for the <code>var</code> property.
     */
    @JspTagVariable(scope = VariableScope.AT_END)
    @JspTagAttribute
    public void setVar(String var) {
        this.var = var;
    }

}
