package com.squeakysand.sling.taglib.resources;

import com.squeakysand.commons.lang.StringUtils;
import com.squeakysand.jsp.tagext.BodyContent;
import com.squeakysand.jsp.tagext.VariableScope;
import com.squeakysand.jsp.tagext.annotations.JspTag;
import com.squeakysand.jsp.tagext.annotations.JspTagAttribute;
import com.squeakysand.jsp.tagext.annotations.JspTagVariable;
import com.squeakysand.sling.taglib.SlingSimpleTagSupport;

import java.io.IOException;
import java.util.Iterator;

import javax.servlet.jsp.JspException;

import org.apache.sling.api.resource.Resource;
import org.apache.sling.api.resource.ResourceResolver;

/**
 * Iterates over the child resources of the specified resource.
 * 
 * @author <a href="http://craigsdickson.com">Craig S. Dickson</a>
 */
@JspTag(bodyContent = BodyContent.SCRIPTLESS)
public class ForEachChildResourceTag extends SlingSimpleTagSupport {

    private String resourcePath;
    private String var;

    /**
     * @see javax.servlet.jsp.tagext.SimpleTagSupport#doTag()
     */
    @Override
    public void doTag() throws JspException, IOException {
        Resource resource = getResource();
        if (StringUtils.isNotNullOrEmpty(resourcePath)) {
            ResourceResolver resourceResolver = getResourceResolver();
            resource = resourceResolver.getResource(resource, resourcePath);
        }
        Iterator<Resource> children = resource.listChildren();
        while (children.hasNext()) {
            Resource child = children.next();
            setPageAttribute(var, child);
            getJspBody().invoke(null);
        }
    }

    /**
     * Accessor for the <code>resourcePath</code> property.
     * 
     * @return the current value of the <code>resourcePath</code> property.
     */
    public String getResourcePath() {
        return resourcePath;
    }

    /**
     * Accessor for the <code>var</code> property.
     * 
     * @return the current value of the <code>var</code> property.
     */
    public String getVar() {
        return var;
    }

    /**
     * The path to the resource relative to the current resource on the page. If this not set, then the current resource
     * will be used.
     * 
     * @param resourcePath
     *            the new value for the <code>resourcePath</code> property.
     */
    @JspTagAttribute(rtexprvalue = true)
    public void setResourcePath(String resourcePath) {
        this.resourcePath = resourcePath;
    }

    /**
     * The name of the page attribute that will hold the child resource.
     * 
     * @param var
     *            the new value for the <code>var</code> property.
     */
    @JspTagAttribute(required = true)
    @JspTagVariable(scope = VariableScope.NESTED, variableClass = Resource.class)
    public void setVar(String var) {
        this.var = var;
    }

}
